/*------------------------------------------------------------------------
 Module:        initialisation.h
 Author:        Pierre Brial
 Project:       Mimee
 State:         version 1.0
 Creation Date: 22 march 2008
 Description:   Global variables and initialisation functions
                used by mimee.c
------------------------------------------------------------------------*/


GtkWidget *window1;
GtkWidget *fixed1;
GtkWidget *X1;
GtkWidget *Y1;
GtkWidget *Z1;
GtkWidget *X2;
GtkWidget *Y2;
GtkWidget *Z2;
GtkWidget *Grille1;
GtkWidget *Grille2;
GtkWidget *Datum1;
GtkWidget *Datum2;
GtkWidget *Ok;
GtkWidget *FIX;
GtkWidget *ZUTM1;
GtkWidget *ZUTM2;
GtkWidget *ZNUTM;
GtkWidget *UTMlock;
GtkWidget *Nord;
GtkWidget *Sud;
GtkWidget *Hemisphere;

gboolean flag1=FALSE;
char *racine;

struct grille grid1,grid2;
char type_grille1[4],type_grille2[4];
struct Datum datum1,datum2;
struct ellipsoide ellips1,ellips2;
FILE *dbIDd,*dbIDg,*logfile;
int indg1=0,indg2=0,indd1=0,indd2=0;
guint32 nrg=4,nrd=0;


/*------------------------------------------------------------------------
 Procedure:     	initdata
 Purpose:       	initialize geodetic databases and load datums and grids names.
 Input:         	None
 Global used :		wgs84 : string "WGS 84"
 					nom_grille[] : grid names
 Output:        	true : databases found
 					false : at least one database was not found
 Globals returned:	dbIDd : datums database ID
 					dbIDg : grids database ID
					ptrd[] : pointers to datums names
					ptrg[] : pointers to grids names
					nrd : number of datums
					nrg : number of grids
------------------------------------------------------------------------*/

gboolean initdata()
{
gboolean b=TRUE;
char ligne[200], nom[20];
char *chemin;

const char nomFDatum[]="Datums.txt";
const char nomFGrilles[]="Grilles.txt";

chemin=catalloc(racine,nomFDatum);
dbIDd=fopen (chemin,"r");
printf ("%s\n",chemin);
free(chemin);

chemin=catalloc(racine,nomFGrilles);
dbIDg=fopen (chemin,"r");
printf ("%s\n",chemin);
free(chemin);

/*Chargement des Noms de Datums*/
printf("Loading databases\n");
if (dbIDd != NULL)
	{
	fgets (ligne, 200, dbIDd);
	while (fgets (ligne, 200, dbIDd)!=NULL)
		{
		sscanf(ligne,"%[^,]",nom);
		if (nom[0]!='#')
			{
			gtk_combo_box_append_text (GTK_COMBO_BOX(Datum1),nom);
			gtk_combo_box_append_text (GTK_COMBO_BOX(Datum2),nom);
			nrd++;
			}
		}
	printf("Nombre de datum = %d\n",nrd);
	fclose(dbIDd);
	}
else b = FALSE;

/*Chargement des Noms de Grilles*/
if (dbIDg != NULL)
	{
	fgets (ligne, 200, dbIDg);
	while (fgets (ligne, 200, dbIDg)!=NULL)
		{
		sscanf(ligne,"%[^,]",nom);
		if (nom[0]!='#')
			{
			gtk_combo_box_append_text (GTK_COMBO_BOX(Grille1),nom);
			gtk_combo_box_append_text (GTK_COMBO_BOX(Grille2),nom);
			nrg++;
			}
		}
	printf("Nombre de grilles = %d\n",nrg);
	fclose(dbIDg);
	}
else b = FALSE;
return b;
}

/*------------------------------------------------------------------------
 Procedure:     	ChargementDonnees
 Purpose:       	load selected datums and grids data
 Input:         	None
 Globals used :     dbIDd : datums database ID
 					dbIDg : grids database ID
					datum1 : datum 1 index
                    datum2 : datum 2 index
					indg1 : grid 1 index
                    indg2 : grid 2 index
 Output:            None
 Globals changed:   ellips1
                    datum1
                    ellips2
                    datum2
                    type_grille1
                    grid1
                    type_grille2
                    grid2
------------------------------------------------------------------------*/
void ChargementDonnees()
	{
	char type_grille[4];
    guint8 i;
    struct ellipsoide ellips;
    struct Datum datum;
    struct grille grid;
    char ligne[200],nom[50];
    
    /* read data of selected datum in datums database */
	void ChargementDatum(guint8 indice)
	     {
	     char *chemin;
	     if (indice==0)
	     	{
	     	ellips.a=6378137;
	     	ellips.e=0.0818191908335;
            memset(&datum,0,sizeof (struct Datum));
            }
            else
            {
            chemin=catalloc(racine,"Datums.txt");
            dbIDd=fopen (chemin,"r");
            for (i=0;i<indice+1;i++)
            	{
             	fgets (ligne, 200, dbIDd);
             	if (ligne[0]=='#') i--;
             	}
            fclose(dbIDd);
            free(chemin);
            sscanf(ligne,"%[^,],%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf",nom,&ellips.a,&ellips.e,&datum.dx,&datum.dy,&datum.dz,&datum.d,&datum.rx,&datum.ry,&datum.rz);
            //printf("%s\n",nom);
            ellips.e=ivf2e(ellips.e);
            }
	     }
        /* read data of selected grid in grids database */
	void ChargementGrille(guint8 indice)
	     {
	     char *chemin;
	     if (indice==4)
	     	{
            strcpy(type_grille,"TM");
            grid.Lat0=0;
            grid.k=0.9996;
            grid.E0=500000;
            grid.N0=0;
            }
		else if (indice<4) strcpy(type_grille,"XX");
		else
			{
			chemin=catalloc(racine,"Grilles.txt");
			dbIDg=fopen (chemin,"r");
			free(chemin);
			for (i=0;i<indice-3;i++)
				{
             	fgets (ligne, 200, dbIDg);
             	if (ligne[0]=='#') i--;
             	}
			fclose(dbIDg);
			sscanf(ligne,"%[^,],%[^,],%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf",nom,type_grille,&grid.Lon0,&grid.Lat0,&grid.k,&grid.E0,&grid.N0,&grid.p1,&grid.p2,&grid.p3);
			grid.Lon0 *= radian;
            grid.Lat0 *= radian;
	        grid.p1 *= radian;
            grid.p2 *= radian;
            grid.p3 *= radian;
			}
		}

        //Chargement des donnees des datums selectionnes
        ChargementDatum(indd1);
        ellips1=ellips;
        datum1=datum;
        ChargementDatum(indd2);
        ellips2=ellips;
        datum2=datum;


        //Chargement des donnees des Grilles selectionnees
        ChargementGrille(indg1);
        strcpy(type_grille1,type_grille);
        grid1=grid;
        ChargementGrille(indg2);
      	strcpy(type_grille2,type_grille);
        grid2=grid;
}

/*------------------------------------------------------------------------
 Procedure:     	DisplayUTMfields
 Purpose:       	hide or show UTM fields
 Input:         	None
 Globals used :		indg1 : grid 1 index
                        indg2 : grid 2 index
 			frm : main form index
 Output:        	None
------------------------------------------------------------------------*/


static void DisplayUTMfields(void)
{
	if (indg1==4)
    	{
    	gtk_widget_show(GTK_WIDGET(Nord));
        gtk_widget_show(GTK_WIDGET(Sud));
        gtk_widget_show(GTK_WIDGET(ZNUTM));
        gtk_widget_show(GTK_WIDGET(ZUTM1));
        }
    else
    	{
        gtk_widget_hide(GTK_WIDGET(Nord));
        gtk_widget_hide(GTK_WIDGET(Sud));
        gtk_widget_hide(GTK_WIDGET(ZNUTM));
        gtk_widget_hide(GTK_WIDGET(ZUTM1));
        }
	if (indg2==4)
		{
        gtk_widget_show(GTK_WIDGET(ZNUTM));
        gtk_widget_show(GTK_WIDGET(UTMlock));
        gtk_widget_show(GTK_WIDGET(ZUTM2));
        gtk_widget_show(GTK_WIDGET(Hemisphere));
        }
	else
		{
        gtk_widget_hide(GTK_WIDGET(ZUTM2));
		gtk_widget_hide(GTK_WIDGET(Hemisphere));
        gtk_widget_hide(GTK_WIDGET(UTMlock));
        }
}

/********************************************************************************/
/*Mise a jour du label des liste Popup et Chargement des donnees correspondantes*/
/********************************************************************************/
static void RefreshPopupLists(void)
{
	//printf("%d\t%d\t%d\t%d\n",indg1,indg2,indd1,indd2);
	flag1=FALSE;
	gtk_combo_box_set_active(GTK_COMBO_BOX(Grille1),indg1);
	gtk_combo_box_set_active(GTK_COMBO_BOX(Grille2),indg2);
	gtk_combo_box_set_active(GTK_COMBO_BOX(Datum1),indd1);
	gtk_combo_box_set_active(GTK_COMBO_BOX(Datum2),indd2);
	flag1=TRUE;
    ChargementDonnees();
}


/*********************************************************/
/*Sauvegarde et Chargement des Données Session précédente*/
/*********************************************************/


void SaveSession()
	{
	FILE *ses;
	char *chemin=catalloc(racine,".session");
	ses=fopen (chemin,"w");
	fprintf(ses,"%d %d %d %d\n",indg1,indg2,indd1,indd2);
	fprintf(ses,"%.8f %.8f %.8f\n",inputd(X1),inputd(Y1),inputd(Z1));
	fprintf(ses,"%.0f %d\n",inputd(ZUTM1),gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(Sud)));
	fclose(ses);
	}


void LoadSession()
	{
	puts("Loading session...");
	char *chemin=catalloc(racine,".session");
	FILE *ses;
	double x1=0.0,y1=0.0,z1=0.0;
	int zutm;
	gboolean hemisphere;
	guint8 decimales=3;
	ses=fopen (chemin,"r");
	free(chemin);
	if (ses)
		{
		fscanf(ses,"%d %d %d %d %lf %lf %lf %d %d",&indg1,&indg2,&indd1,&indd2,&x1,&y1,&z1,&zutm,&hemisphere);
		RefreshPopupLists();
		if (indg1<3) decimales=8;
		if (x1!=0) printdx(X1,x1,decimales);
        if (y1!=0) printdx(Y1,y1,decimales);
        if (z1!=0) printdx(Z1,z1,3);
        printi(ZUTM1,zutm);
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(Sud),hemisphere);
        fclose(ses);
        }
    DisplayUTMfields();
    }


void StartLogFile()
	{
	char *chemin=catalloc(racine,"logfile.txt");
	printf("%s\n",chemin);
	logfile=fopen (chemin,"w");
	free(chemin);
	}

